<?php

namespace LitePress\User_Module;

use WP_Http;
use WP_REST_Request;
use WP_REST_Response;

/**
 * 用户信息数据类
 *
 * @version 1.0
 * @date 2021-10-11
 */
final class User_Info {

    /**
     * 用户的 API Token
     *
     * @var string
     */
    private $token = '';

    /**
     * 用户的显示名称
     *
     * @var string
     */
    private $display_name = '';

    /**
     * 用户的登录名
     *
     * @var string
     */
    private $login_name = '';

    /**
     * @return string
     */
    public function get_login_name() {
        return $this->login_name;
    }

    /**
     * @param string $login_name
     */
    public function set_login_name( string $login_name ) {
        $this->login_name = $login_name;
    }

    public function get_token() {
        return $this->token;
    }

    public function set_token( $token ) {
        $this->token = $token;
    }

    public function get_display_name() {
        return $this->display_name;
    }

    public function set_display_name( $display_name ) {
        $this->display_name = $display_name;
    }

}

/**
 * 用户模块类
 *
 * @version 1.0
 * @date 2021-10-11
 *
 * 该类用于在插件中方便的集成 litepress.cn 用户认证体系
 */
class User_Module {

    const base_url = 'https://api.litepress.cn';
    const login_api = self::base_url . '/auth/v1/token';
    const option_key = 'lp_user_info';
    private static $instance;

    private function __construct() {
        add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ) );
    }

    public function plugins_loaded() {
        // 注册 API 端点
        add_action( 'rest_api_init', array( self::get_instance(), 'api_route' ) );

        // 输出 JS
        add_action( 'admin_footer', array( self::get_instance(), 'script' ) );
    }

    public static function get_instance() {
        if ( ! ( self::$instance instanceof User_Module ) ) {
            self::$instance = new User_Module();
        }

        return self::$instance;
    }

    public function script() {
        echo <<<JS
<script>
const api_domain = location.origin + "/wp-json";

jQuery(function ($) {
$('.wp-pwd button').click(function()
    {
        $(this).find("span").toggleClass('dashicons-visibility dashicons-hidden');
       type = $(this).prev().attr("type");

       if (type==="password"){
           $(this).prev().prop('type', 'text')
    }
        else {
           $(this).prev().prop('type', 'password')
       }
    });

    /*登录流程*/
    $(".wp_login_btn").click(function () {
        username = $("#user_login").val();
        password = $("#user_pass").val();
        $.ajax({
            url:    api_domain + "/litepress/user/login",
            type: "post",
            dataType: "JSON",
            data: {
                username: username,
                password: password,
            },
            beforeSend: function (data) {
                $("#wp-submit").before("<span class=\"spinner is-active\"></span>")
            },
            success: function (data) {
                $("#login_error").remove("");
                window.location.reload();
            },
            error: function (error) {
                login_val = $("#user_login").val();
                pass_val = $("#user_pass").val();
                if (login_val === "" && pass_val === "") {
                    $("#login_error,.spinner").remove("");
                    $(".navbar-brand").after("<div id=\"login_error\">\t<strong>错误</strong>：用户名字段为空。<br>" +
                        "\t<strong>错误</strong>：密码字段为空。<br>" +
                        "</div>")
                } else {
                    $("#login_error,.spinner").remove("");
                    $(".navbar-brand").after("<div id=\"login_error\">"+error.responseJSON.message + "  <a href=\"https://litepress.cn/wp-login.php?action=lostpassword\">忘记密码？</a><br>" +
                        "</div>")
                }
                console.log(error);
            },
            complete: function () {
            },
        })
    })
 
    /*注销流程*/
    $(".logout").click(function () {
        $.ajax({
            url: api_domain + "/litepress/user/logout",
            type: "post",
            dataType: "JSON",
            success: function (data) {
                window.location.reload();
            },
            error: function (error) {
                console.log(error);
            },
            complete: function () {
            },
        })
    })
})
</script>
JS;
    }

    public function api_route() {
        /**
         * 用户登录 API 接口
         */
        register_rest_route( 'litepress/user', 'login', array(
            'methods'             => 'POST',
            'callback'            => array( self::$instance, 'user_login' ),
            'permission_callback' => '__return_true',
        ) );

        /**
         * 用户注销登录 API 接口
         */
        register_rest_route( 'litepress/user', 'logout', array(
            'methods'             => 'POST',
            'callback'            => array( self::$instance, 'user_logout' ),
            'permission_callback' => '__return_true',
        ) );
    }

    /**
     * 用户注销接口函数
     */
    public function user_logout() {
        $this->clear_login_status();

        return $this->success( '注销成功' );
    }

    /**
     * 清除登录状态
     */
    public function clear_login_status() {
        delete_option( self::option_key );
    }

    /**
     * 成功时的 API 返回
     */
    private function success( $message = '', $data = array() ) {
        return new WP_REST_Response( array(
            'code'    => 0,
            'message' => $message,
            'data'    => $data,
        ) );
    }

    /**
     * 用户登录接口函数
     */
    public function user_login( WP_REST_Request $request ) {
        $username = sanitize_text_field( $request->get_param( 'username' ) );
        $password = sanitize_text_field( $request->get_param( 'password' ) );

        if ( empty( $username ) ) {
            return $this->error( 'username 字段不可为空' );
        }

        if ( empty( $password ) ) {
            return $this->error( 'password 字段不可为空' );
        }

        $args = array(
            'headers' => array(
                'Accept-Encoding' => 'gzip',
            ),
            'body'    => array(
                'username' => $username,
                'password' => $password,
            )
        );
        $r    = wp_remote_post( self::login_api, $args );
        if ( is_wp_error( $r ) ) {
            return $this->error( "登录失败，接口返回：{$r->get_error_message()}" );
        }

        $status_code = wp_remote_retrieve_response_code( $r );

        if ( 403 === $status_code ) {
            return $this->error( "登录失败，账号或密码错误。" );
        }

        if ( WP_Http::OK !== $status_code ) {
            return $this->error( "登录失败，接口返回状态码：{$status_code}" );
        }

        $body = wp_remote_retrieve_body( $r );

        $body = json_decode( $body, true );
        if ( ! isset( $body['token'] ) || ! isset( $body['user_display_name'] ) ) {
            return $this->error( "登录失败，接口返回未知数据格式，希望可以向我们反馈" );
        }

        $user_info = new User_Info();
        $user_info->set_token( $body['token'] );
        $user_info->set_display_name( $body['user_display_name'] );
        $user_info->set_login_name( $body['user_nicename'] );

        update_option( self::option_key, $user_info, true );

        return $this->success( '登录成功' );
    }

    /**
     * 失败时的 API 返回
     */
    private function error( $message = '' ) {
        return new WP_REST_Response( array(
            'code'    => - 1,
            'message' => $message,
        ), 500 );
    }

    /**
     * 输出用于登录的前端组件
     */
    public function print_login_view() {
        if ( $this->is_logged() ) {
            $user_name = $this->get_user_info()->get_display_name();
            echo <<<HTML
已登录为：$user_name 　<a class="button logout">注销</a>
HTML;
        } else {
            add_thickbox();

            echo <<<HTML
 <a class="thickbox button button-primary" title="登录" href="#TB_inline?height=300&width=300&inlineId=login-1">
            登录 LitePress.cn 账号
          </a>
<div id="login-1" style="display:none;">
        <div class="login">
            <a class="navbar-brand" href="https://litepress.cn" target="_blank" one-link-mark="yes">
                <img src="https://litepress.cn/wp-content/uploads/2021/05/logo.svg" alt="LitePress 社区"
                     width="200"></a>
          <p>
            <label for="user_login">用户名或电子邮箱地址</label>
            <input type="text" name="log" id="user_login" class="input" value="" size="20" autocapitalize="none">
          </p>

          <div class="user-pass-wrap">
            <label for="user_pass">密码</label>
            <div class="wp-pwd">
              <input type="password" name="pwd" id="user_pass" class="input password-input" value="" size="20">
              <button type="button" class="button button-secondary wp-hide-pw hide-if-no-js" data-toggle="0"
                      aria-label="显示密码">
                <span class="dashicons dashicons-visibility" aria-hidden="true"></span>
              </button>
            </div>
          </div>
          <section class="submit">
              <p id="nav">
                  <a rel="nofollow" href="https://litepress.cn/wp-login.php?action=register" target="_blank"
                     one-link-mark="yes">注册</a> | <a href="https://litepress.cn/wp-login.php?action=lostpassword"
                                                     target="_blank" one-link-mark="yes">忘记密码？</a>
              </p>
            <input type="submit" name="wp-submit" id="wp-submit" class="button button-primary button-large wp_login_btn"
                   value="登录">
          </section>

        </div>
      </div>
<script>

function show_login_box() {
  jQuery(function ($) {
      if($(".thickbox.button").length > 0) {
        tb_show("","#TB_inline?height=400&amp;width=300&amp;inlineId=login-1",false);
      }
  })
}

setTimeout(show_login_box, 1000);
</script>
<style>
.login #login_error {
    
        border-left: 4px solid #00a0d2;
        border-left-color: #dc3232;
    padding: 12px;
}
.login {
    margin-top: 20px;
    margin-left: 0;
    padding: 26px 24px 34px;
    font-weight: 400;
    overflow: hidden;
    background: #fff;
}
.navbar-brand {
    text-align: center;
    display: block;
    margin-bottom: 30px;
}
.login form .input,
.login input[type=password],
.login input[type=text] {
    font-size: 24px;
    line-height: 1.33333333;
    width: 100%;
    border-width: .0625rem;
    padding: .1875rem .3125rem;
    margin: 0 6px 16px 0;
    min-height: 40px;
    max-height: none
}
#TB_ajaxWindowTitle {
    display: none;
}

.tb-close-icon {
    color: #fff !important;
}

#TB_closeWindowButton {
    right: -40px !important;
}

#TB_closeWindowButton:focus .tb-close-icon {
    box-shadow: none !important;
}
#TB_window #TB_title {
    background: none;
    border-bottom: 0;
    position: absolute;
    right: 0;
}
.login .button.wp-hide-pw:active {
    background: 0 0;
    box-shadow: none;
    transform: none;
}
.login .button.wp-hide-pw .dashicons {
    width: 1.25rem;
    height: 1.25rem;
    top: .25rem;
}
.login .button.wp-hide-pw {
    background: 0 0;
    border: 1px solid transparent;
    box-shadow: none;
    font-size: 14px;
    line-height: 2;
    width: 2.5rem;
    height: 2.5rem;
    min-width: 40px;
    min-height: 40px;
    margin: 0;
    padding: 5px 9px;
    position: absolute;
    right: 0;
    top: 0;
}
.login .wp-pwd {
    position: relative;
}
section.submit {
    display: flex;
    align-items: center;
    margin-top: 20px;
}
.login #backtoblog, .login #nav {
    font-size: 13px;
}
.login .button-primary {
    margin-left: auto;
}
.login * {
    margin: 0;
    padding: 0;
}
</style>

HTML;
        }
    }

    /**
     * 检查用户是否已登录
     */
    public function is_logged() {
        return ! empty( get_option( self::option_key ) );
    }

    /**
     * 从 Session 尝试获取当前用户的信息
     */
    public function get_user_info() {
        return get_option( self::option_key, new User_Info() );
    }

}
