<?php

namespace LitePress\LP_Translate;

use LitePress\LP_Translate\MT\MT;
use LitePress\User_Module\User_Module;
use Loco_error_AdminNotices;
use Loco_fs_File;
use Loco_mvc_ViewParams;
use Loco_package_Bundle;
use WP_Http;

define( 'LP_TR_VERSION', '2.5.4-beta8' );
define( 'LP_TR_ROOT_URL', plugins_url( '', loco_plugin_self() ) . '/lp' );

add_action( 'admin_enqueue_scripts', function () {
    add_thickbox();

    wp_enqueue_style( 'lp-translate', LP_TR_ROOT_URL . '/assets/lp-translate.css', array(), LP_TR_VERSION );
    wp_enqueue_script( 'lp-translate', LP_TR_ROOT_URL . '/assets/lp-translate.js', array(), LP_TR_VERSION );
} );

/**
 * 载入机器翻译模块
 */
require __DIR__ . '/mt.php';

/**
 * 载入用户模块
 */
require __DIR__ . '/class-user-module.php';
User_Module::get_instance();

/**
 * 载入翻译项目数据类
 */
require __DIR__ . '/class-project.php';

/**
 * 载入辅助函数
 */
require __DIR__ . '/helper.php';

/**
 * 载入 API 端点
 */
require __DIR__ . '/api/server.php';

/**
 * 当项目翻译编辑页面加载时检查该项目是否已收录
 */
add_action( 'wp_loaded', function () {
    // 只有在前端通过 Ajax 触发 fsConnect 路由时才执行，这样可以保证在页面上只执行一次
    if ( ! isset( $_GET['path'] ) || ! isset( $_GET['domain'] ) || ! isset( $_GET['bundle'] ) ) {
        return;
    }

    // 判断是否被收录
    if ( exist_cloud( $_GET['domain'] ) ) {
        return;
    }

    /**
     * 获取项目详情
     */
    if ( stristr( $_GET['path'], 'plugins/' ) ) {
        $local_project = get_local_project_info( 'plugin', $_GET['bundle'] );
    } else {
        $local_project = get_local_project_info( 'theme', $_GET['bundle'] );
    }

    // 获取用户 Token
    $login_token = User_Module::get_instance()->get_user_info()->get_token();

    Loco_error_AdminNotices::warn( "该项目当前未被 litepress.cn 收录，所有依赖云端处理的功能都将无法使用。你可以点击右侧链接向我们申请收录，我会在工作日的 6 个小时内审核所有申请" )
                           ->addLink( "https://litepress.cn/translate/projects/others/-new?textdomain={$_GET['domain']}&project_name={$local_project->get_name()}&login_token={$login_token}", '申请托管' )
                           ->addLink( '/wp-json/lp-tr/project/status/refresh', '刷新状态' );
}, 100 );


/**
 * 每当页面被访问时尝试获取当前用户所管理的项目
 */
add_action( 'wp_loaded', function () {
    if ( ! User_Module::get_instance()->is_logged() ) {
        return false;
    }

    $user = User_Module::get_instance()->get_user_info();

    // 如果已经存在缓存的数据则不再获取
    //$cloud_projects = get_transient( "lp_cloud_projects_by_{$user->get_login_name()}" );
    if ( empty( $cloud_projects ) ) {

        // 获取当前安装的所有插件及主题的 paths;
        $paths = array();

        if ( ! function_exists( 'get_plugins' ) ) {
            require ABSPATH . '/wp-admin/includes/plugin.php';
        }

        $plugins = get_plugins();
        foreach ( $plugins as $plugin ) {
            if ( ! key_exists( 'TextDomain', $plugin ) || empty( $plugin['TextDomain'] ) ) {
                continue;
            }

            $text_domain = $plugin['TextDomain'];
            $paths[]     = "plugins/{$text_domain}/body";
        }

        if ( ! function_exists( 'wp_get_themes' ) ) {
            require ABSPATH . '/wp-admin/includes/theme.php';
        }
        $themes = wp_get_themes();
        foreach ( $themes as $k => $v ) {
            $paths[] = "themes/{$k}/{$k}";
        }

        // 请求接口获取用户所管理的项目
        $args     = array(
            'headers' => array(
                'Authorization' => "Bearer {$user->get_token()}",
            ),
        );
        $base_url = 'https://api.litepress.cn/translate/projects';
        $url      = add_query_arg( array(
            'paths' => json_encode( $paths ),
        ), $base_url );
        $result   = wp_remote_get( $url, $args );

        $status_code = wp_remote_retrieve_response_code( $result );
        if ( WP_Http::OK !== $status_code ) {
            // 如果状态码为 403 需要清除一下本地的登陆状态，可能是 Token 过期了
            if ( WP_Http::FORBIDDEN === $status_code ) {
                User_Module::get_instance()->clear_login_status();

                Loco_error_AdminNotices::warn( "LP Translate 的登陆状态已过期" );

                return false;
            }

            Loco_error_AdminNotices::warn( "从 litepress.cn 获取当前管理的项目时接口返回了意料之外的状态码：{$status_code}" );

            return false;
        }

        if ( is_wp_error( $result ) ) {
            Loco_error_AdminNotices::warn( "从 litepress.cn 获取当前管理的项目时接口返回了错误消息：{$result->get_error_message()}" );

            return false;
        }

        $body = wp_remote_retrieve_body( $result );

        $projects = json_decode( $body, true );
        if ( isset( $projects['error'] ) ) {
            Loco_error_AdminNotices::warn( "从 litepress.cn 获取当前管理的项目时接口返回了错误消息：{$projects['error']}" );

            return false;
        }

        $cloud_projects = array();

        foreach ( $projects['projects'] ?? array() as $project ) {
            $project_obj = new Project();
            $project_obj->set_name( $project['name'] );
            $project_obj->set_textdomain( $project['slug'] );
            $project_obj->set_path( $project['path'] );
            $project_obj->set_permission( $project['permission'] );
            $project_obj->set_version( $project['version'] );

            list( $type, $domain, $branch ) = explode( '/', $project['path'] );

            $cloud_projects[ $domain ] = $project_obj;
        }

        set_transient( "lp_cloud_projects_by_{$user->get_login_name()}", $cloud_projects, 86400 );
    }

    return true;
} );

// 当用户同步原文时返回从 litepress.cn 加载的文件
add_filter( 'lp_translate_sync_file', function ( $now, $textdomain, $path ) {
    if ( ! stristr( $path, 'plugins/' ) && ! stristr( $path, 'themes/' ) ) {
        return $now;
    }

    $type   = stristr( $path, 'plugins/' ) ? 'plugins' : 'themes';
    $branch = 'plugins' === $type ? 'body' : $textdomain;

    $tmp_path = download_url( "https://litepress.cn/translate/projects/$type/$textdomain/$branch/zh-cn/default/export-translations/?filters[term]&filters[term_scope]=scope_any&filters[status]=current_or_waiting_or_fuzzy_or_untranslated&filters[user_login]&format=po" );
    if ( is_wp_error( $tmp_path ) && 'http_404' === $tmp_path->get_error_code() ) {
        // 如果未找到项目则尝试去 others 目录中搜寻
        $tmp_path = download_url( "https://litepress.cn/translate/projects/others/$textdomain/$branch/zh-cn/default/export-translations/?filters[term]&filters[term_scope]=scope_any&filters[status]=current_or_waiting_or_fuzzy_or_untranslated&filters[user_login]&format=po" );
        if ( is_wp_error( $tmp_path ) && 'http_404' === $tmp_path->get_error_code() ) {
            Loco_error_AdminNotices::warn( '当前项目未被 litepress.cn 收录，已从本地为你提取翻译，你可以向我们 <a href="https://litepress.cn/translate/projects/others/-new" target="_blank">建议收录</a> 该项目' );

            return $now;
        }
    }

    if ( is_wp_error( $tmp_path ) ) {
        Loco_error_AdminNotices::warn( '从 litepress.cn 同步翻译时出错，接口返回错误信息：' . $path->get_error_message() );

        return $now;
    }

    $full_path = WP_CONTENT_DIR . "/{$path}";
    if ( ! is_writable( $full_path ) ) {
        Loco_error_AdminNotices::warn( '从 litepress.cn 同步翻译时出错，无法将翻译文件移动到：' . $path );

        return $now;
    }
    rename( $tmp_path, $full_path );

    $file = new Loco_fs_File( $path );
    $base = loco_constant( 'WP_CONTENT_DIR' );
    $file->normalize( $base );

    return $file;
}, 10, 3 );

// 当用户保存翻译时同步回 litepress.cn
add_action( 'lp_translate_save_file', 'LitePress\LP_Translate\translate_save_file', 10, 3 );
function translate_save_file( $textdomain, $path, $bundle ): bool {
    if ( ! stristr( $path, 'plugins/' ) && ! stristr( $path, 'themes/' ) ) {
        return false;
    }

    if ( ! User_Module::get_instance()->is_logged() ) {
        Loco_error_AdminNotices::warn( "你当前未登录 LitePress.cn 账号，故我们未向云端同步你的翻译。" );

        return false;
    }

    /**
     * WP_Http 似乎不支持上传文件，所以这里只能用原生的 CURL 写了
     */

    $full_path = WP_CONTENT_DIR . "/$path";

    list ( $type, $bundle ) = explode( '.', $bundle, 2 );
    $branch       = 'plugin' === $type ? 'body' : $textdomain;
    $project_path = "{$type}s/$textdomain/$branch";

    if ( function_exists( 'curl_file_create' ) ) { // php 5.5+
        $cFile = curl_file_create( $full_path );
    } else {
        $cFile = '@' . realpath( $full_path );
    }

    // 获取本地的项目版本号并一同提交给 API
    $local_project = get_local_project_info( $type, $bundle );

    $post = array(
        'project_path' => $project_path,
        'po_file'      => $cFile,
        'version'      => $local_project->get_version(),
    );

    $ch = curl_init();
    curl_setopt( $ch, CURLOPT_URL, 'https://api.litepress.cn/gp/translations' );
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1 );
    curl_setopt( $ch, CURLOPT_POST, 1 );
    curl_setopt( $ch, CURLOPT_POSTFIELDS, $post );
    curl_setopt( $ch, CURLOPT_HTTPHEADER, array(
        'Authorization: Bearer ' . User_Module::get_instance()->get_user_info()->get_token(),
    ) );

    $result      = curl_exec( $ch );
    $status_code = curl_getinfo( $ch, CURLINFO_HTTP_CODE );
    curl_close( $ch );

    if ( WP_Http::OK !== $status_code ) {
        // 如果状态码为 403 需要清除一下本地的登陆状态，可能是 Token 过期了
        if ( WP_Http::FORBIDDEN === $status_code ) {
            User_Module::get_instance()->clear_login_status();

            Loco_error_AdminNotices::warn( "LP Translate 的登陆状态已过期" );

            return false;
        }

        Loco_error_AdminNotices::warn( "向 litepress.cn 上传翻译时出错，接口返回了意料之外的状态码：{$status_code}" );

        return false;
    }


    $targets = json_decode( $result, true );
    if ( isset( $targets['error'] ) ) {
        Loco_error_AdminNotices::warn( "向 litepress.cn 上传翻译时出错：{$targets['error']}" );

        return false;
    }

    return true;
}

add_filter( 'lp_bundle_param', function ( Loco_mvc_ViewParams $args, Loco_package_Bundle $bundle ) {
    $cloud_projects = get_cloud_projects();

    $login_token = User_Module::get_instance()->get_user_info()->get_token();

    foreach ( $bundle->getDomains() as $domain => $obj ) {
        if ( isset( $cloud_projects[ $domain ] ) ) {
            /**
             * @var $project Project
             */
            $project = $cloud_projects[ $domain ];

            $args['cloud_version'] = $project->get_version();
            if ( empty( $args['cloud_version'] ) ) {
                $args['cloud_version'] = '未填写';
            }

            switch ( $project->get_permission() ) {
                case 'approve':
                    $args['permission'] = '审批者';
                    break;
                case 'manage':
                    $args['permission'] = '管理者';
                    break;
                default:
                    $args['permission'] = "<a class='button apply-approve' href='https://litepress.cn/forums/topic/21858' target='_blank' domain='{$domain}'>申请权限</a>";
            }
        } else {
            $args['cloud_version'] = "<a class='button' href='https://litepress.cn/translate/projects/others/-new?textdomain={$domain}&project_name={$bundle->getName()}&login_token={$login_token}' target='_blank'>申请托管</a>";
        }
    }

    return $args;
}, 10, 2 );

/**
 * 在机器翻译工作之前对比一下插件的本地版本和云端版本，如果本地更新的话需要先上传，然后再调用机器翻译
 * 这是因为翻译引擎只对已收录的字符串才有效，所以为了防止本地项目版本大幅落后云端版本导致无法使用机器翻译的情况，就需要在翻译前先上传原文
 * 当然，只有当用户拥有 manage 权限时原文才会被平台录入。
 */
add_action( 'pre_translate_word', function () {
    if ( ! User_Module::get_instance()->is_logged() ) {
        return false;
    }

    // 因为 API 请求是不会附加插件信息的，所以从请求来源的 URL 中尝试解析
    $referer_query = get_query_for_referer();

    $path   = $referer_query['path'] ?? '';
    $bundle = $referer_query['bundle'] ?? '';

    $type = stristr( $path, 'plugins' ) ? 'plugin' : 'theme';

    $local_project = get_local_project_info( $type, $bundle );
    $cloud_project = get_project_for_cloud( $local_project->get_textdomain() );
    if ( empty( $local_project ) || empty( $cloud_project ) ) {
        return false;
    }

    // 只有在当前用户对云端项目拥有 manage 权限时才执行后续操作
    if ( 'manage' !== $cloud_project->get_permission() ) {
        return false;
    }

    if ( ! empty( $cloud_project->get_version() ) && version_compare( $cloud_project->get_version(), $local_project->get_version(), '>=' ) ) {
        return false;
    }

    return translate_save_file( $local_project->get_textdomain(), $path, "{$type}.{$bundle}" );
} );

$mt = new MT();
add_filter( 'loco_api_providers', array( $mt, 'translator_filter_apis' ), 10, 1 );
add_action( 'loco_api_ajax', array( $mt, 'translator_ajax_init' ), 0, 0 );

/**
 * 修改语言包存储的首选位置
 */
add_filter( 'lp_lang_preferred_storage_location', function ( $default, $sortable ) {
    // 这里将首选位置设置为系统目录
    $system = array();
    foreach ( $sortable as $item ) {
        if ( 'translation' === $item['systype'] ) {
            $system = $item;
        }
    }
    $system['checked'] = 'checked';

    return $system;
}, 10, 2 );
